/*==============================================================================
*
*  LOGIC CORE:          			
*  MODULE NAME:         Key_Board()
*  COMPANY:             芯航线电子工作室
*                       xiaomeige.taobao.com
*	author:					小梅哥
*	author QQ：528369266  
*	芯航线FPGA技术支持群 472607506
*  REVISION HISTORY:  
*
*    Revision 1.0  25/12/2014     Description: Initial Release.
*
*  FUNCTIONAL DESCRIPTION:
*
*	FileType	：RTL
*
*
===============================================================================*/

module Key_Board(Clk,Rst_n,Key_Board_Row_i,Key_Board_Col_o,Key_Flag,Key_Value,beep,SEG7_SCLK,SEG7_RCLK,SEG7_DIO);

	input Clk;
	input Rst_n;
	
	input [3:0] Key_Board_Row_i;
	output reg [3:0] Key_Board_Col_o;
	
	output reg Key_Flag;
	output reg [3:0] Key_Value;
	output beep;
	output SEG7_SCLK;
	output SEG7_RCLK;
	output SEG7_DIO;
	
	reg [19:0] counter1;
	reg En_Cnt;
	reg [10:0]state;
	reg [3:0]Key_Board_Row_r;
	reg Cnt_Done;
	reg [3:0]Col_Tmp;
	reg [7:0]Key_Value_tmp;
	reg Key_Flag_r;
	
	localparam 
		IDEL         = 11'b00000000001,
		P_FILTER     = 11'b00000000010,
		READ_ROW_P   = 11'b00000000100,
		SCAN_C0      = 11'b00000001000,
		SCAN_C1      = 11'b00000010000,
		SCAN_C2      = 11'b00000100000,
		SCAN_C3      = 11'b00001000000,
		PRESS_RESULT = 11'b00010000000,
		WAIT_R       = 11'b00100000000,
		R_FILTER     = 11'b01000000000,
		READ_ROW_R   = 11'b10000000000;
	
	probe probe_inst0(
		.probe  ( Key_Value ),
		.source (  )
	);
	
	probe probe_inst1(
		.probe  ( state ),
		.source (  )
	);
	
	ibeep ibeep(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.En_beep(Key_Flag),
		.Beep(beep)
	);
	
	HEX_top HEX_top(
		.Clk       (Clk),
		.Rst_n     (Rst_n),
		.disp_data ({28'd0,Key_Value}),
		.SEG7_SCLK (SEG7_SCLK),
		.SEG7_RCLK (SEG7_RCLK),
		.SEG7_DIO  (SEG7_DIO)
	);
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		counter1 <= 20'd0;
	else if(En_Cnt)begin
		if(counter1 == 20'd999999)
			counter1 <= 20'd0;
		else
			counter1 <= counter1 + 1'b1;
	end
	else
		counter1 <= 20'd0;
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Cnt_Done <= 1'b0;
	else if(counter1 == 20'd999999)
		Cnt_Done <= 1'b1;
	else
		Cnt_Done <= 1'b0;
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		En_Cnt <= 1'b0;
		state <= IDEL;
		Key_Board_Col_o <= 4'b0000;
		Col_Tmp <= 4'd0;
		Key_Flag_r <= 1'b0;
		Key_Value_tmp <= 8'd0;
		Key_Board_Row_r <= 4'b1111;
	end
	else begin
		case(state)
			IDEL:
				if(~&Key_Board_Row_i)begin
					En_Cnt <= 1'b1;
					state <= P_FILTER;			
				end
				else begin
					En_Cnt <= 1'b0;
					state <= IDEL;			
				end
				
			P_FILTER:
				if(Cnt_Done) begin
					En_Cnt <= 1'b0;
					state <= READ_ROW_P;
				end
				else begin
					En_Cnt <= 1'b1;
					state <= P_FILTER;				
				end
			
			READ_ROW_P:
				if(~Key_Board_Row_i)begin
					Key_Board_Row_r <= Key_Board_Row_i;
					state <= SCAN_C0;	
					Key_Board_Col_o <= 4'b1110;
				end
				else begin
					state <= IDEL;
					Key_Board_Col_o <= 4'b0000;
				end
			
			SCAN_C0:
				begin
					state <= SCAN_C1;	
					Key_Board_Col_o <= 4'b1101;
					if(~&Key_Board_Row_i)						
						Col_Tmp <= 4'b0001;
					else
						Col_Tmp <= 4'b0000;
				end
				
			SCAN_C1:
				begin
					state <= SCAN_C2;	
					Key_Board_Col_o <= 4'b1011;
					if(~&Key_Board_Row_i)						
						Col_Tmp <= Col_Tmp | 4'b0010;
					else
						Col_Tmp <= Col_Tmp;				
				end
				
			SCAN_C2:
				begin
					state <= SCAN_C3;	
					Key_Board_Col_o <= 4'b0111;
					if(~&Key_Board_Row_i)
						Col_Tmp <= Col_Tmp | 4'b0100;
					else
						Col_Tmp <= Col_Tmp;				
				end
			
			SCAN_C3:
				begin
					state <= PRESS_RESULT;	
					if(~&Key_Board_Row_i)
						Col_Tmp <= Col_Tmp | 4'b1000;
					else
						Col_Tmp <= Col_Tmp;				
				end
				
			PRESS_RESULT:
				begin
					state <= WAIT_R;
					Key_Board_Col_o <= 4'b0000;
					if(((Key_Board_Row_r[0] + Key_Board_Row_r[1] + Key_Board_Row_r[2] + Key_Board_Row_r[3]) == 4'd3) && 
						((Col_Tmp[0] + Col_Tmp[1] + Col_Tmp[2] + Col_Tmp[3]) == 4'd1))begin
						Key_Flag_r <= 1'b1;
						Key_Value_tmp <= {Key_Board_Row_r,Col_Tmp};
					end
					else begin
						Key_Flag_r <= 1'b0;
						Key_Value_tmp <= Key_Value_tmp;
					end
				end
			WAIT_R:
				begin
					Key_Flag_r <= 1'b0;
					if(&Key_Board_Row_i)begin
						En_Cnt <= 1'b1;
						state <= R_FILTER;
					end
					else begin
						state <= WAIT_R;
						En_Cnt <= 1'b0;
					end
				end
			
			R_FILTER:					
					if(Cnt_Done) begin
						En_Cnt <= 1'b0;
						state <= READ_ROW_R;
					end
					else begin
						En_Cnt <= 1'b1;
						state <= R_FILTER;				
					end
				
			READ_ROW_R:
				if(&Key_Board_Row_i)
					state <= IDEL;
				else begin
					En_Cnt <= 1'b1;
					state <= R_FILTER;				
				end
			default:state <= IDEL;
		endcase	
	end
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		Key_Value <= 4'd0;
	end
	else if(Key_Flag_r)begin
		case(Key_Value_tmp)
			8'b1110_0001 : Key_Value <= 4'd0;
			8'b1110_0010 :	Key_Value <= 4'd1;
			8'b1110_0100 : Key_Value <= 4'd2;
			8'b1110_1000 : Key_Value <= 4'd3;
			
			8'b1101_0001 : Key_Value <= 4'd4;
			8'b1101_0010 : Key_Value <= 4'd5;
			8'b1101_0100 : Key_Value <= 4'd6;
			8'b1101_1000 : Key_Value <= 4'd7;
			
			8'b1011_0001 : Key_Value <= 4'd8;
			8'b1011_0010 : Key_Value <= 4'd9;
			8'b1011_0100 : Key_Value <= 4'd10;
			8'b1011_1000 : Key_Value <= 4'd11;
			
			8'b0111_0001 : Key_Value <= 4'd12;
			8'b0111_0010 : Key_Value <= 4'd13;
			8'b0111_0100 : Key_Value <= 4'd14;
			8'b0111_1000 : Key_Value <= 4'd15;
			default:begin Key_Value <= Key_Value;end
		endcase
	end
	
	always@(posedge Clk)
		Key_Flag <= Key_Flag_r;
	
endmodule
